"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _spec = require("./spec");

var _is = require("./is");

var _private = require("./private");

var _memoize = require("lodash/memoize");

var _memoize2 = _interopRequireDefault(_memoize);

var _lib = require("./lib");

var _bug = require("./bug");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = (0, _private.createPrivateStorage)();

var __target__ = Symbol("target");

function isFloat16Array(target) {
    return target instanceof Float16Array;
}

function assertFloat16Array(target) {
    if (!isFloat16Array(target)) {
        throw new TypeError("This is not a Float16Array");
    }
}

function isDefaultFloat16ArrayMethods(target) {
    return typeof target === "function" && defaultFloat16ArrayMethods.has(target);
}

function copyToArray(float16bits) {
    var length = float16bits.length;

    var array = new Array(length);
    for (var i = 0; i < length; ++i) {
        array[i] = (0, _lib.convertNumber)(float16bits[i]);
    }

    return array;
}

// proxy handler
var handler = {
    get(target, key) {
        var wrapper = null;
        if (!_bug.isTypedArrayIndexedPropertyWritable) {
            wrapper = target;
            target = _(wrapper).target;
        }

        if ((0, _is.isNumberKey)(key)) {
            return (0, _lib.convertNumber)(Reflect.get(target, key));
        } else {
            var ret = wrapper !== null && Reflect.has(wrapper, key) ? Reflect.get(wrapper, key) : Reflect.get(target, key);

            if (typeof ret !== "function") return ret;

            // TypedArray methods can't be called by Proxy
            var proxy = _(ret).proxy;

            if (proxy === undefined) {
                proxy = _(ret).proxy = new Proxy(ret, {
                    apply(func, thisArg, args) {

                        // peel off proxy                        
                        if (isFloat16Array(thisArg) && isDefaultFloat16ArrayMethods(func)) return Reflect.apply(func, _bug.isProxyAbleToBeWeakMapKey ? _(thisArg).target : thisArg[__target__], args);

                        return Reflect.apply(func, thisArg, args);
                    }
                });
            }

            return proxy;
        }
    },

    set(target, key, value) {
        var wrapper = null;
        if (!_bug.isTypedArrayIndexedPropertyWritable) {
            wrapper = target;
            target = _(wrapper).target;
        }

        if ((0, _is.isNumberKey)(key)) {
            return Reflect.set(target, key, (0, _lib.roundToFloat16Bits)(value));
        } else {
            // frozen object can't change prototype property
            if (wrapper !== null && (!Reflect.has(target, key) || Object.isFrozen(wrapper))) {
                return Reflect.set(wrapper, key, value);
            } else {
                return Reflect.set(target, key, value);
            }
        }
    }
};

if (!_bug.isTypedArrayIndexedPropertyWritable) {
    handler.getPrototypeOf = wrapper => Reflect.getPrototypeOf(_(wrapper).target);
    handler.setPrototypeOf = (wrapper, prototype) => Reflect.setPrototypeOf(_(wrapper).target, prototype);

    handler.defineProperty = (wrapper, key, descriptor) => {
        var target = _(wrapper).target;
        return !Reflect.has(target, key) || Object.isFrozen(wrapper) ? Reflect.defineProperty(wrapper, key, descriptor) : Reflect.defineProperty(target, key, descriptor);
    };
    handler.deleteProperty = (wrapper, key) => {
        var target = _(wrapper).target;
        return Reflect.has(wrapper, key) ? Reflect.deleteProperty(wrapper, key) : Reflect.deleteProperty(target, key);
    };

    handler.has = (wrapper, key) => Reflect.has(wrapper, key) || Reflect.has(_(wrapper).target, key);

    handler.isExtensible = wrapper => Reflect.isExtensible(wrapper);
    handler.preventExtensions = wrapper => Reflect.preventExtensions(wrapper);

    handler.getOwnPropertyDescriptor = (wrapper, key) => Reflect.getOwnPropertyDescriptor(wrapper, key);
    handler.ownKeys = wrapper => Reflect.ownKeys(wrapper);
}

class Float16Array extends Uint16Array {

    constructor(input, byteOffset, length) {

        // input Float16Array
        if (isFloat16Array(input)) {
            super(_bug.isProxyAbleToBeWeakMapKey ? _(input).target : input[__target__]);

            // 22.2.1.3, 22.2.1.4 TypedArray, Array, ArrayLike, Iterable
        } else if (input !== null && typeof input === "object" && !(0, _is.isArrayBuffer)(input)) {
            // if input is Iterable, get Array
            var array = (0, _is.isArrayLike)(input) ? input : [...input];

            var _length = array.length;
            super(_length);

            for (var i = 0; i < _length; ++i) {
                // super (Uint16Array)
                this[i] = (0, _lib.roundToFloat16Bits)(array[i]);
            }

            // 22.2.1.2, 22.2.1.5 primitive, ArrayBuffer
        } else {
            switch (arguments.length) {
                case 0:
                    super();
                    break;

                case 1:
                    super(input);
                    break;

                case 2:
                    super(input, byteOffset);
                    break;

                default:
                    super(input, byteOffset, length);
            }
        }

        var proxy = void 0;

        if (_bug.isTypedArrayIndexedPropertyWritable) {
            proxy = new Proxy(this, handler);
        } else {
            var wrapper = Object.create(null);
            _(wrapper).target = this;
            proxy = new Proxy(wrapper, handler);
        }

        // proxy private storage
        if (_bug.isProxyAbleToBeWeakMapKey) {
            _(proxy).target = this;
        } else {
            this[__target__] = this;
        }

        // this private storage
        _(this).proxy = proxy;

        return proxy;
    }

    // static methods
    static from(src) {
        if ((arguments.length <= 1 ? 0 : arguments.length - 1) === 0) return new Float16Array(Uint16Array.from(src, _lib.roundToFloat16Bits).buffer);

        var mapFunc = arguments.length <= 1 ? undefined : arguments[1];
        var thisArg = arguments.length <= 2 ? undefined : arguments[2];

        return new Float16Array(Uint16Array.from(src, function (val) {
            for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                args[_key - 1] = arguments[_key];
            }

            return (0, _lib.roundToFloat16Bits)(mapFunc.call(this, val, ...args));
        }, thisArg).buffer);
    }

    static of() {
        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
        }

        return new Float16Array(args);
    }

    // iterate methods
    *[Symbol.iterator]() {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = super[Symbol.iterator]()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var val = _step.value;

                yield (0, _lib.convertNumber)(val);
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }
    }

    keys() {
        return super.keys();
    }

    *values() {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
            for (var _iterator2 = super.values()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var val = _step2.value;

                yield (0, _lib.convertNumber)(val);
            }
        } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                    _iterator2.return();
                }
            } finally {
                if (_didIteratorError2) {
                    throw _iteratorError2;
                }
            }
        }
    }

    *entries() {
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
            for (var _iterator3 = super.entries()[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                var _ref = _step3.value;

                var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

                var i = _ref2[0];
                var val = _ref2[1];

                yield [i, (0, _lib.convertNumber)(val)];
            }
        } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                    _iterator3.return();
                }
            } finally {
                if (_didIteratorError3) {
                    throw _iteratorError3;
                }
            }
        }
    }

    // functional methods
    map(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        var array = [];
        for (var i = 0, l = this.length; i < l; ++i) {
            var val = (0, _lib.convertNumber)(this[i]);
            array.push(callback.call(thisArg, val, i, _(this).proxy));
        }

        return new Float16Array(array);
    }

    filter(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        var array = [];
        for (var i = 0, l = this.length; i < l; ++i) {
            var val = (0, _lib.convertNumber)(this[i]);

            if (callback.call(thisArg, val, i, _(this).proxy)) array.push(val);
        }

        return new Float16Array(array);
    }

    reduce(callback) {
        assertFloat16Array(this);

        var val = void 0,
            start = void 0;

        if ((arguments.length <= 1 ? 0 : arguments.length - 1) === 0) {
            val = (0, _lib.convertNumber)(this[0]);
            start = 1;
        } else {
            val = arguments.length <= 1 ? undefined : arguments[1];
            start = 0;
        }

        for (var i = start, l = this.length; i < l; ++i) {
            val = callback(val, (0, _lib.convertNumber)(this[i]), i, _(this).proxy);
        }

        return val;
    }

    reduceRight(callback) {
        assertFloat16Array(this);

        var val = void 0,
            start = void 0;

        var length = this.length;
        if ((arguments.length <= 1 ? 0 : arguments.length - 1) === 0) {
            val = (0, _lib.convertNumber)(this[length - 1]);
            start = length - 1;
        } else {
            val = arguments.length <= 1 ? undefined : arguments[1];
            start = length;
        }

        for (var i = start; i--;) {
            val = callback(val, (0, _lib.convertNumber)(this[i]), i, _(this).proxy);
        }

        return val;
    }

    forEach(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        for (var i = 0, l = this.length; i < l; ++i) {
            callback.call(thisArg, (0, _lib.convertNumber)(this[i]), i, _(this).proxy);
        }
    }

    find(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        for (var i = 0, l = this.length; i < l; ++i) {
            var value = (0, _lib.convertNumber)(this[i]);
            if (callback.call(thisArg, value, i, _(this).proxy)) return value;
        }
    }

    findIndex(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        for (var i = 0, l = this.length; i < l; ++i) {
            var value = (0, _lib.convertNumber)(this[i]);
            if (callback.call(thisArg, value, i, _(this).proxy)) return i;
        }

        return -1;
    }

    every(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        for (var i = 0, l = this.length; i < l; ++i) {
            if (!callback.call(thisArg, (0, _lib.convertNumber)(this[i]), i, _(this).proxy)) return false;
        }

        return true;
    }

    some(callback) {
        assertFloat16Array(this);

        var thisArg = arguments.length <= 1 ? undefined : arguments[1];

        for (var i = 0, l = this.length; i < l; ++i) {
            if (callback.call(thisArg, (0, _lib.convertNumber)(this[i]), i, _(this).proxy)) return true;
        }

        return false;
    }

    // change element methods
    set(input) {
        assertFloat16Array(this);

        var offset = arguments.length <= 1 ? undefined : arguments[1];

        var float16bits = void 0;

        // input Float16Array
        if (isFloat16Array(input)) {
            float16bits = _bug.isProxyAbleToBeWeakMapKey ? _(input).target : input[__target__];

            // input others
        } else {
            var array = (0, _is.isArrayLike)(input) ? input : [...input];
            var length = array.length;

            float16bits = new Uint16Array(length);
            for (var i = 0, l = array.length; i < l; ++i) {
                float16bits[i] = (0, _lib.roundToFloat16Bits)(array[i]);
            }
        }

        super.set(float16bits, offset);
    }

    reverse() {
        assertFloat16Array(this);

        super.reverse();

        return _(this).proxy;
    }

    fill(value) {
        assertFloat16Array(this);

        for (var _len3 = arguments.length, opts = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
            opts[_key3 - 1] = arguments[_key3];
        }

        super.fill((0, _lib.roundToFloat16Bits)(value), ...opts);

        return _(this).proxy;
    }

    copyWithin(target, start) {
        assertFloat16Array(this);

        for (var _len4 = arguments.length, opts = Array(_len4 > 2 ? _len4 - 2 : 0), _key4 = 2; _key4 < _len4; _key4++) {
            opts[_key4 - 2] = arguments[_key4];
        }

        super.copyWithin(target, start, ...opts);

        return _(this).proxy;
    }

    sort() {
        assertFloat16Array(this);

        var compareFunction = arguments.length <= 0 ? undefined : arguments[0];

        if (compareFunction === undefined) {
            compareFunction = _spec.defaultCompareFunction;
        }

        var _convertNumber = (0, _memoize2.default)(_lib.convertNumber);

        super.sort((x, y) => compareFunction(_convertNumber(x), _convertNumber(y)));

        return _(this).proxy;
    }

    // copy element methods
    slice() {
        assertFloat16Array(this);

        var float16bits = void 0;

        // V8, SpiderMonkey, JavaScriptCore throw TypeError
        try {
            float16bits = super.slice(...arguments);
        } catch (e) {
            if (e instanceof TypeError) {
                var uint16 = new Uint16Array(this.buffer, this.byteOffset, this.length);
                float16bits = uint16.slice(...arguments);
            } else {
                throw e;
            }
        }

        return new Float16Array(float16bits.buffer);
    }

    subarray() {
        assertFloat16Array(this);

        var float16bits = void 0;

        // SpiderMonkey, JavaScriptCore throw TypeError
        try {
            float16bits = super.subarray(...arguments);
        } catch (e) {
            if (e instanceof TypeError) {
                var uint16 = new Uint16Array(this.buffer, this.byteOffset, this.length);
                float16bits = uint16.subarray(...arguments);
            } else {
                throw e;
            }
        }

        return new Float16Array(float16bits.buffer, float16bits.byteOffset, float16bits.length);
    }

    // contains methods
    indexOf(element) {
        assertFloat16Array(this);

        var length = this.length;

        var from = (0, _spec.ToInteger)(arguments.length <= 1 ? undefined : arguments[1]);

        if (from < 0) {
            from += length;
            if (from < 0) from = 0;
        }

        for (var i = from, l = length; i < l; ++i) {
            if ((0, _lib.convertNumber)(this[i]) === element) return i;
        }

        return -1;
    }

    lastIndexOf(element) {
        assertFloat16Array(this);

        var length = this.length;

        var from = (0, _spec.ToInteger)(arguments.length <= 1 ? undefined : arguments[1]);

        from = from === 0 ? length : from + 1;

        if (from >= 0) {
            from = from < length ? from : length;
        } else {
            from += length;
        }

        for (var i = from; i--;) {
            if ((0, _lib.convertNumber)(this[i]) === element) return i;
        }

        return -1;
    }

    includes(element) {
        assertFloat16Array(this);

        var length = this.length;

        var from = (0, _spec.ToInteger)(arguments.length <= 1 ? undefined : arguments[1]);

        if (from < 0) {
            from += length;
            if (from < 0) from = 0;
        }

        var isNaN = Number.isNaN(element);
        for (var i = from, l = length; i < l; ++i) {
            var value = (0, _lib.convertNumber)(this[i]);

            if (isNaN && Number.isNaN(value)) return true;

            if (value === element) return true;
        }

        return false;
    }

    // string methods
    join() {
        assertFloat16Array(this);

        var array = copyToArray(this);

        return array.join(...arguments);
    }

    toLocaleString() {
        assertFloat16Array(this);

        var array = copyToArray(this);

        return array.toLocaleString(...arguments);
    }

    get [Symbol.toStringTag]() {
        if (isFloat16Array(this)) return "Float16Array";
    }

}

exports.default = Float16Array;
var Float16Array$prototype = Float16Array.prototype;

var defaultFloat16ArrayMethods = new WeakSet();
var _iteratorNormalCompletion4 = true;
var _didIteratorError4 = false;
var _iteratorError4 = undefined;

try {
    for (var _iterator4 = Reflect.ownKeys(Float16Array$prototype)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
        var key = _step4.value;

        var val = Float16Array$prototype[key];
        if (typeof val === "function") defaultFloat16ArrayMethods.add(val);
    }
} catch (err) {
    _didIteratorError4 = true;
    _iteratorError4 = err;
} finally {
    try {
        if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
        }
    } finally {
        if (_didIteratorError4) {
            throw _iteratorError4;
        }
    }
}

module.exports = exports["default"];