// ftp://ftp.fox-toolkit.org/pub/fasthalffloatconversion.pdf

"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.roundToFloat16Bits = roundToFloat16Bits;
exports.convertNumber = convertNumber;
var buffer = new ArrayBuffer(4);
var floatView = new Float32Array(buffer);
var uint32View = new Uint32Array(buffer);

var baseTable = new Uint32Array(512);
var shiftTable = new Uint32Array(512);

for (var i = 0; i < 256; ++i) {
    var e = i - 127;

    // very small number (0, -0)
    if (e < -27) {
        baseTable[i | 0x000] = 0x0000;
        baseTable[i | 0x100] = 0x8000;
        shiftTable[i | 0x000] = 24;
        shiftTable[i | 0x100] = 24;

        // small number (denorm)
    } else if (e < -14) {
        baseTable[i | 0x000] = 0x0400 >> -e - 14;
        baseTable[i | 0x100] = 0x0400 >> -e - 14 | 0x8000;
        shiftTable[i | 0x000] = -e - 1;
        shiftTable[i | 0x100] = -e - 1;

        // normal number
    } else if (e <= 15) {
        baseTable[i | 0x000] = e + 15 << 10;
        baseTable[i | 0x100] = e + 15 << 10 | 0x8000;
        shiftTable[i | 0x000] = 13;
        shiftTable[i | 0x100] = 13;

        // large number (Infinity, -Infinity)
    } else if (e < 128) {
        baseTable[i | 0x000] = 0x7c00;
        baseTable[i | 0x100] = 0xfc00;
        shiftTable[i | 0x000] = 24;
        shiftTable[i | 0x100] = 24;

        // stay (NaN, Infinity, -Infinity)        
    } else {
        baseTable[i | 0x000] = 0x7c00;
        baseTable[i | 0x100] = 0xfc00;
        shiftTable[i | 0x000] = 13;
        shiftTable[i | 0x100] = 13;
    }
}

/**
 * round a number to a half float number bits.
 * @param {number} num
 */
function roundToFloat16Bits(num) {
    floatView[0] = num;

    var f = uint32View[0];
    var e = f >> 23 & 0x1ff;
    return baseTable[e] + ((f & 0x007fffff) >> shiftTable[e]);
}

var mantissaTable = new Uint32Array(2048);
var exponentTable = new Uint32Array(64);
var offsetTable = new Uint32Array(64);

// mantissa
mantissaTable[0] = 0;
for (var _i = 1; _i < 1024; ++_i) {
    var m = _i << 13; // zero pad mantissa bits
    var _e = 0; // zero exponent

    // normalized
    while ((m & 0x00800000) === 0) {
        _e -= 0x00800000; // decrement exponent
        m <<= 1;
    }

    m &= ~0x00800000; // clear leading 1 bit
    _e += 0x38800000; // adjust bias

    mantissaTable[_i] = m | _e;
}
for (var _i2 = 1024; _i2 < 2048; ++_i2) {
    mantissaTable[_i2] = 0x38000000 + (_i2 - 1024 << 13);
}

// exponent
exponentTable[0] = 0;
for (var _i3 = 1; _i3 < 31; ++_i3) {
    exponentTable[_i3] = _i3 << 23;
}
exponentTable[31] = 0x47800000;
exponentTable[32] = 0x80000000;
for (var _i4 = 33; _i4 < 63; ++_i4) {
    exponentTable[_i4] = 0x80000000 + (_i4 - 32 << 23);
}
exponentTable[63] = 0xc7800000;

// offset
offsetTable[0] = 0;
for (var _i5 = 1; _i5 < 64; ++_i5) {
    if (_i5 === 32) {
        offsetTable[_i5] = 0;
    } else {
        offsetTable[_i5] = 1024;
    }
}

/**
 * convert a half float number bits to a number.
 * @param {number} h - half float number bits
 */
function convertNumber(h) {
    var m = h >> 10;
    uint32View[0] = mantissaTable[offsetTable[m] + (h & 0x3ff)] + exponentTable[m];
    return floatView[0];
}